@@ -4,6 +4,8 @@ struct abc {
  long a;
  long b;
  long c;
+ long d;
+ long e;
 };
  
 struct abc foo2(){}
