@@ -39,3 +39,9 @@ int rvarr(int n, int a[restrict static n]) {
   return a[2];
 }
 
+typedef struct {
+  int f;
+} T __attribute__ ((__aligned__));
+
+// CHECK: struct __attribute__((visibility("default"))) S;
+struct __attribute__((visibility("default"))) S;
